radar_data = [
    {"name": "Matcha Latte", "value1": 78, "value2": 6, "value3": 85},
    {"name": "Milk Tea", "value1": 17, "value2": 10, "value3": 63},
    {"name": "Cheese Cocoa", "value1": 18, "value2": 15, "value3": 65},
    {"name": "Walnut Brownie", "value1": 9, "value2": 71, "value3": 16},
]
