indicator_data_variant_1 = [
    {
        "description": "",
        "title": "Estado",
        "value": "Abierto",
        "align": "center",
    },
    {
        "description": "",
        "title": "Price ($)",
        "value": "455",
        "color": "success",
    },
    {
        "description": "this is a description",
        "title": "Volumen Changed!",
        "value": "41153",
    },
    {
        "description": "Crazy description",
        "title": "Cambio €/$",
        "value": "1.1946",
    },
]

indicator_data_variant_2 = [
    {
        "color": "success",
        "variant": "contained",
        "description": "This indicator has a Link",
        "targetPath": "/indicators/indicator/1",
        "title": "Target Indicator",
        "align": "left",
        "value": "500€",
    },
    {
        "color": "warning",
        "backgroundImage": "https://images.unsplash.com/photo-1535957998253-26ae1ef29506?ixlib=rb-1.2.1&ixid=MnwxMjA3fD"
        "B8MHxwaG90by1wYWdlfHx8fGVufDB8fHx8&auto=format&fit=crop&w=736&q=80",
        "variant": "outlined",
        "description": "This has a background",
        "title": "Super cool indicator",
        "align": "left",
        "value": "Value",
    },
    {
        "color": "error",
        "variant": "outlined",
        "description": "This hasn't got any icons",
        "title": "Error indicator",
        "align": "left",
        "value": "Value",
        "icon": "Line/download",
    },
    {
        "color": "caution",
        "variant": "contained",
        "description": "Aligned to right and full of icons",
        "title": "Multiple cases",
        "align": "right",
        "value": "Value",
        "icon": "Line/download",
        "bigIcon": "Line/calendar",
    },
]
