import datetime as dt
import random

table_with_labels_data = [
    {
        "date": dt.date(2021, 1, 1),
        "x": 5,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "A",
        "filtB": "Z",
        "name": "Ana",
        "name2": "Ana",
        "name3": "Ana",
    },
    {
        "date": dt.date(2021, 1, 2),
        "x": 6,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "B",
        "filtB": "Z",
        "name": "Laura",
        "name2": "Laura",
        "name3": "Laura",
    },
    {
        "date": dt.date(2021, 1, 3),
        "x": 4,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "A",
        "filtB": "W",
        "name": "Audrey",
        "name2": "Audrey",
        "name3": "Audrey",
    },
    {
        "date": dt.date(2021, 1, 4),
        "x": 7,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "B",
        "filtB": "W",
        "name": "Jose",
        "name2": "Jose",
        "name3": "Jose",
    },
    {
        "date": dt.date(2021, 1, 5),
        "x": 3,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "A",
        "filtB": "Z",
        "name": "Jorge",
        "name2": "Jorge",
        "name3": "Jorge",
    },
    {
        "date": dt.date(2021, 1, 1),
        "x": 5,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "A",
        "filtB": "Z",
        "name": "Ana",
        "name2": "Ana",
        "name3": "Ana",
    },
    {
        "date": dt.date(2021, 1, 2),
        "x": 6,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "B",
        "filtB": "Z",
        "name": "Laura",
        "name2": "Laura",
        "name3": "Laura",
    },
    {
        "date": dt.date(2021, 1, 3),
        "x": 4,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "A",
        "filtB": "W",
        "name": "Audrey",
        "name2": "Audrey",
        "name3": "Audrey",
    },
    {
        "date": dt.date(2021, 1, 4),
        "x": 7,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "B",
        "filtB": "W",
        "name": "Jose",
        "name2": "Jose",
        "name3": "Jose",
    },
    {
        "date": dt.date(2021, 1, 5),
        "x": 3,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "A",
        "filtB": "Z",
        "name": "Jorge",
        "name2": "Jorge",
        "name3": "Jorge",
    },
    {
        "date": dt.date(2021, 1, 1),
        "x": 5,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "A",
        "filtB": "Z",
        "name": "Ana",
        "name2": "Ana",
        "name3": "Ana",
    },
    {
        "date": dt.date(2021, 1, 2),
        "x": 6,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "B",
        "filtB": "Z",
        "name": "Laura",
        "name2": "Laura",
        "name3": "Laura",
    },
    {
        "date": dt.date(2021, 1, 3),
        "x": 4,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "A",
        "filtB": "W",
        "name": "Audrey",
        "name2": "Audrey",
        "name3": "Audrey",
    },
    {
        "date": dt.date(2021, 1, 4),
        "x": 7,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "B",
        "filtB": "W",
        "name": "Jose",
        "name2": "Jose",
        "name3": "Jose",
    },
    {
        "date": dt.date(2021, 1, 5),
        "x": 3,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "A",
        "filtB": "Z",
        "name": "Jorge",
        "name2": "Jorge",
        "name3": "Jorge",
    },
    {
        "date": dt.date(2021, 1, 1),
        "x": 5,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "A",
        "filtB": "Z",
        "name": "Ana",
        "name2": "Ana",
        "name3": "Ana",
    },
    {
        "date": dt.date(2021, 1, 2),
        "x": 6,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "B",
        "filtB": "Z",
        "name": "Laura",
        "name2": "Laura",
        "name3": "Laura",
    },
    {
        "date": dt.date(2021, 1, 3),
        "x": 4,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "A",
        "filtB": "W",
        "name": "Audrey",
        "name2": "Audrey",
        "name3": "Audrey",
    },
    {
        "date": dt.date(2021, 1, 4),
        "x": 7,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "B",
        "filtB": "W",
        "name": "Jose",
        "name2": "Jose",
        "name3": "Jose",
    },
    {
        "date": dt.date(2021, 1, 5),
        "x": 3,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "A",
        "filtB": "Z",
        "name": "Jorge",
        "name2": "Jorge",
        "name3": "Jorge",
    },
    {
        "date": dt.date(2021, 1, 1),
        "x": 5,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "A",
        "filtB": "Z",
        "name": "Ana",
        "name2": "Ana",
        "name3": "Ana",
    },
    {
        "date": dt.date(2021, 1, 2),
        "x": 6,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "B",
        "filtB": "Z",
        "name": "Laura",
        "name2": "Laura",
        "name3": "Laura",
    },
    {
        "date": dt.date(2021, 1, 3),
        "x": 4,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "A",
        "filtB": "W",
        "name": "Audrey",
        "name2": "Audrey",
        "name3": "Audrey",
    },
    {
        "date": dt.date(2021, 1, 4),
        "x": 7,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "B",
        "filtB": "W",
        "name": "Jose",
        "name2": "Jose",
        "name3": "Jose",
    },
    {
        "date": dt.date(2021, 1, 5),
        "x": 3,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "A",
        "filtB": "Z",
        "name": "Jorge",
        "name2": "Jorge",
        "name3": "Jorge",
    },
    {
        "date": dt.date(2021, 1, 1),
        "x": 5,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "A",
        "filtB": "Z",
        "name": "Ana",
        "name2": "Ana",
        "name3": "Ana",
    },
    {
        "date": dt.date(2021, 1, 2),
        "x": 6,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "B",
        "filtB": "Z",
        "name": "Laura",
        "name2": "Laura",
        "name3": "Laura",
    },
    {
        "date": dt.date(2021, 1, 3),
        "x": 4,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "A",
        "filtB": "W",
        "name": "Audrey",
        "name2": "Audrey",
        "name3": "Audrey",
    },
    {
        "date": dt.date(2021, 1, 4),
        "x": 7,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "B",
        "filtB": "W",
        "name": "Jose",
        "name2": "Jose",
        "name3": "Jose",
    },
    {
        "date": dt.date(2021, 1, 5),
        "x": 3,
        "y": round(100 * random.random(), 10),
        "z": round(100 * random.random(), 1),
        "a": round(100 * random.random(), 1),
        "filtA": "A",
        "filtB": "Z",
        "name": "Jorge",
        "name2": "Jorge",
        "name3": "Jorge",
    },
]
