import datetime as dt

table_data = [
    {
        "date": dt.date(2021, 1, 1),
        "x": 5,
        "y": 5,
        "filtA": "A",
        "filtB": "Z",
        "name": "Ana",
        "web": "https://www.google.com",
        "webName": "Google",
        "web2": "https://www.youtube.com",
        "webName2": "Youtube",
        "modal": "Modal 1",
    },
    {
        "date": dt.date(2021, 1, 2),
        "x": 6,
        "y": 5,
        "filtA": "B",
        "filtB": "Z",
        "name": "Laura",
        "web": "https://www.facebook.com",
        "webName": "Facebook",
        "web2": "https://www.twitter.com",
        "webName2": "Twitter",
        "modal": "Modal 2",
    },
    {
        "date": dt.date(2021, 1, 3),
        "x": 4,
        "y": 5,
        "filtA": "A",
        "filtB": "W",
        "name": "Audrey",
        "web": "https://www.twitter.com",
        "webName": "Twitter",
        "web2": "https://www.instagram.com",
        "webName2": "Instagram",
        "modal": "Modal 3",
    },
    {
        "date": dt.date(2021, 1, 4),
        "x": 7,
        "y": 5,
        "filtA": "B",
        "filtB": "W",
        "name": "Jose",
        "web": "https://www.instagram.com",
        "webName": "Instagram",
        "web2": "https://www.linkedin.com",
        "webName2": "Linkedin",
        "modal": "Modal 1",
    },
    {
        "date": dt.date(2021, 1, 5),
        "x": 3,
        "y": 5,
        "filtA": "A",
        "filtB": "Z",
        "name": "Jorge",
        "web": "https://www.linkedin.com",
        "webName": "Linkedin",
        "web2": "https://www.google.com",
        "webName2": "Google",
        "modal": "Modal 2",
    },
]
