horizontal_bar_data = [
    {"name": "Financiado", "value": 1321},
    {"name": "Product name 2", "value": 1400},
    {"name": "Product name 3", "value": 1499},
    {"name": "Policy tenure", "value": 1550},
    {"name": "Family size", "value": 1663},
    {"name": "Acquisitive power", "value": 1995},
    {"name": "Geo", "value": 2712},
    {"name": "Modelo", "value": 3521},
    {"name": "Año", "value": 4643},
    {"name": "Offer", "value": 5426},
]
