bar_and_line_data = [
    {"day": "Mon", "Evaporation": 2.0, "Precipitation": 2.6, "Temperature": 2.0},
    {"day": "Tue", "Evaporation": 4.9, "Precipitation": 5.9, "Temperature": 2.2},
    {"day": "Wed", "Evaporation": 7.0, "Precipitation": 9.0, "Temperature": 3.3},
    {"day": "Thu", "Evaporation": 23.2, "Precipitation": 26.4, "Temperature": 4.5},
    {"day": "Fri", "Evaporation": 25.6, "Precipitation": 28.7, "Temperature": 6.3},
    {"day": "Sat", "Evaporation": 76.7, "Precipitation": 70.7, "Temperature": 10.2},
    {"day": "Sun", "Evaporation": 135.6, "Precipitation": 175.6, "Temperature": 20.3},
]
