sunburst_data = [
    {
        "name": "Root 1",
        "children": [
            {
                "name": "Children A",
                "value": 15,
                "children": [
                    {"name": "Children A1", "value": 2},
                    {
                        "name": "Children AA1",
                        "value": 5,
                        "children": [{"name": "Children AAA1", "value": 2}],
                    },
                    {"name": "Children A2", "value": 4},
                ],
            },
            {
                "name": "Children B",
                "value": 10,
                "children": [
                    {"name": "Children B1", "value": 5},
                    {"name": "Children B2", "value": 1},
                ],
            },
        ],
    },
    {
        "name": "Root 2",
        "children": [
            {
                "name": "Children A1",
                "children": [
                    {"name": "Children AA1", "value": 1},
                    {"name": "Children AA2", "value": 2},
                ],
            }
        ],
    },
]
