funnel_data = [
    {"value": 60, "name": "Third"},
    {"value": 40, "name": "Fourth"},
    {"value": 20, "name": "Fifth"},
    {"value": 80, "name": "Second"},
    {"value": 100, "name": "First"},
]
