tree_data = [
    {
        "name": "root",
        "value": 35,
        "children": [
            {
                "name": "Child A",
                "value": 9,
                "children": [
                    {"name": "Child A1", "value": 23},
                    {"name": "Child A2", "value": 72},
                    {"name": "Child A3", "value": 93},
                ],
            },
            {
                "name": "Child B",
                "value": 56,
                "children": [
                    {"name": "Child B1", "value": 39},
                    {"name": "Child B2", "value": 61},
                    {"name": "Child B3", "value": 71},
                ],
            },
            {
                "name": "Child C",
                "value": 100,
                "children": [
                    {"name": "Child C1", "value": 19},
                    {"name": "Child C2", "value": 66},
                    {"name": "Child C3", "value": 47},
                ],
            },
        ],
    }
]
