html_data = (
    "<p style='background-color: #daf4f0';>"
    "Comparing the results of predictions that happened previous "
    "periods vs reality, so that you can measure the accuracy of our predictor"
    "</p>"
)
