indicator_color_by_value_data = [
    {
        "align": "center",
        "title": "Positive case",
        "value": 10,
    },
    {
        "align": "center",
        "title": "Negative case",
        "value": -10,
    },
    {
        "align": "center",
        "title": "Neutral case",
        "value": 0,
    },
]
